package cn.rankin.tradeservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.OrderDetailStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.trade.dto.OrderDetailSearchDTO;
import cn.rankin.data.api.trade.entity.OrderDetail;
import cn.rankin.data.api.trade.entity.OrderGoods;
import cn.rankin.tradeservice.code.TradeServiceAPICode;
import cn.rankin.tradeservice.repository.OrderDetailRepository;
import cn.rankin.tradeservice.repository.OrderGoodsRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;

@Service
public class OrderDetailService {

    @Autowired
    private OrderDetailRepository orderDetailRepository;

    @Autowired
    private OrderGoodsRepository orderGoodsRepository;

    public APIResult<OrderDetail> getOrder(String detailId) {
        OrderDetail orderDetail = orderDetailRepository.find(detailId);
        if (orderDetail == null) {
            return APIResult.error(TradeServiceAPICode.NOT_EXISTS);
        }

        List<OrderGoods> orderGoodsList = orderGoodsRepository.findByOrderId(detailId);
        orderDetail.setGoods(orderGoodsList);
        return APIResult.ok(orderDetail);
    }

    public APIResult<Page<OrderDetail>> search(OrderDetailSearchDTO searchDTO) {
        Specification<OrderDetail> specification = new Specification<OrderDetail>() {
            @Override
            public Predicate toPredicate(Root<OrderDetail> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
                return OrderDetailService.this.toPredicate(root, query, cb, searchDTO);
            }
        };

        Long count = orderDetailRepository.count(specification);
        Page<OrderDetail> page = new Page<>(count, searchDTO.getPageNo(), searchDTO.getPageSize());

        Long start = page.getStart();
        if (count == 0) {
            return APIResult.ok(page);
        }

        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);
        List<OrderDetail> orderList = orderDetailRepository.find(specification, start, page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(orderList);

        return APIResult.ok(page);
    }

    public Predicate toPredicate(Root<OrderDetail> root, CriteriaQuery<?> query, CriteriaBuilder cb, OrderDetailSearchDTO searchDTO) {
        // 存放查询条件
        List<Predicate> list = new ArrayList<Predicate>();
        Predicate tmp = null;

        List<String> userList = searchDTO.getUid();
        if (!CollectionUtils.isEmpty(userList)) {
            tmp = root.get("uid").in(userList);
            list.add(tmp);
        }

        String merchantId = searchDTO.getMerchantId();
        if (!StringUtils.isEmpty(merchantId)) {
            tmp = cb.equal(root.get("merchantId"), merchantId);
            list.add(tmp);
        }

        OrderDetailStatusEnum status = searchDTO.getStatus();
        if (status != null) {
            tmp = cb.equal(root.get("orderStatus"), status);
            list.add(tmp);
        }

        Date beginTime = searchDTO.getBeginTime();
        if (beginTime != null) {
            tmp = cb.greaterThanOrEqualTo(root.get("gmtCreated"), beginTime);
            list.add(tmp);
        }

        Date endTime = searchDTO.getEndTime();
        if (endTime != null) {
            tmp = cb.lessThanOrEqualTo(root.get("gmtCreated"), endTime);
            list.add(tmp);
        }
        // where条件
        return list.isEmpty() ? null : cb.and(list.toArray(new Predicate[list.size()]));
    }
}
